@@ -18,7 +18,7 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
-    $Id: orbit-idl-main.c,v 1.26 2003/07/25 15:36:37 danw Exp $
+    $Id: orbit-idl-main.c,v 1.27 2003/09/30 11:55:14 michael Exp $
 
 ***************************************************************************/
 
@@ -139,7 +139,7 @@ struct poptOption options[] = {
 int main(int argc, const char *argv[])
 {
   poptContext pcon;
-  int rc;
+  int rc, retval = 0;
   const char *arg;
   OIDL_Run_Info rinfo;
 
@@ -215,9 +215,10 @@ int main(int argc, const char *argv[])
     rinfo.input_filename = g_strdup (arg); /* g_path_get_basename(arg); - what !? */
     if (!orbit_idl_to_backend(arg, &rinfo)) {
       g_warning("%s compilation failed", arg);
+      retval = 1;
     }
     g_free(rinfo.input_filename);
   }
 
-  return 0;
+  return retval;
 }
