@@ -36,7 +36,7 @@
 #include "orb-core-private.h"
 #include "orbit-debug.h"
 
-int ORBit_small_flags = ORBIT_SMALL_FAST_LOCALS;
+int ORBit_small_flags = 0;
 
 gpointer
 ORBit_small_alloc (CORBA_TypeCode tc)
@@ -382,27 +382,19 @@ typedef enum {
 	MARSHAL_CLEAN
 } DeMarshalRetType;
 
-typedef enum {
-	RECV_BUFFER_RELEASE,
-	RECV_BUFFER_KEEP
-} DeMarshalRecvRelease;
-
 static DeMarshalRetType
 orbit_small_demarshal (CORBA_Object           obj,
 		       GIOPConnection       **cnx,
-		       GIOPMessageQueueEntry *mqe,
+		       GIOPRecvBuffer        *recv_buffer,
 		       CORBA_Environment     *ev,
 		       gpointer               ret,
 		       ORBit_IMethod         *m_data,
-		       gpointer              *args,
-		       DeMarshalRecvRelease   recv_release)
+		       gpointer              *args)
 {
 	gpointer        data, p;
 	CORBA_TypeCode  tc;
-	GIOPRecvBuffer *recv_buffer;
 	CORBA_ORB       orb = obj->orb;
 
-	recv_buffer = giop_recv_buffer_get (mqe);
 	if (!recv_buffer) {
 		dprintf (MESSAGES, "No recv buffer ...\n");
 		return MARSHAL_SYS_EXCEPTION_INCOMPLETE;
@@ -525,9 +517,6 @@ orbit_small_demarshal (CORBA_Object           obj,
 			tprintf (" )");
 	}
 
-	if (recv_release == RECV_BUFFER_RELEASE)
-		giop_recv_buffer_unuse (recv_buffer);
-
 	return MARSHAL_CLEAN;
 
  demarshal_exception:
@@ -543,17 +532,11 @@ orbit_small_demarshal (CORBA_Object           obj,
 		*cnx = ORBit_handle_location_forward (recv_buffer, obj);
 		tprintf (" Exception: forward");
 
-		if (recv_release == RECV_BUFFER_RELEASE)
-			giop_recv_buffer_unuse (recv_buffer);
-
 		return MARSHAL_RETRY;
 	} else {
 		ORBit_handle_exception_array (
 			recv_buffer, ev, &m_data->exceptions, obj->orb);
 
-		if (recv_release == RECV_BUFFER_RELEASE)
-			giop_recv_buffer_unuse (recv_buffer);
-
 #ifdef G_ENABLE_DEBUG
 		if (_orbit_debug_flags & ORBIT_DEBUG_TRACES) {
 			if (ev->_major == CORBA_SYSTEM_EXCEPTION)
@@ -578,8 +561,14 @@ ORBit_small_invoke_stub_n (CORBA_Object        object,
 			   CORBA_Context       ctx,
 			   CORBA_Environment  *ev)
 {
-	/* FIXME: validate on the sequence length ? */
-	ORBit_small_invoke_stub (object, &methods->_buffer[index], ret, args, ctx, ev);
+	if (index < 0 || index > methods->_length) {
+		dprintf (MESSAGES, "Cannot invoke OOB method (%ld,%lu)\n",
+			 index, (gulong)methods->_length);
+		CORBA_exception_set_system (ev, ex_CORBA_NO_IMPLEMENT,
+					    CORBA_COMPLETED_NO);
+
+	} else
+		ORBit_small_invoke_stub (object, &methods->_buffer[index], ret, args, ctx, ev);
 }
 
 void
@@ -595,6 +584,8 @@ ORBit_small_invoke_stub (CORBA_Object       obj,
 	GIOPConnection         *cnx = NULL;
 	GIOPMessageQueueEntry   mqe;
 	ORBit_OAObject          adaptor_obj;
+	GIOPRecvBuffer         *recv_buffer = NULL;
+	CORBA_Object            xt_proxy = CORBA_OBJECT_NIL;
 
 	if (!obj) {
 		dprintf (MESSAGES, "Cannot invoke method on null object\n");
@@ -606,12 +597,20 @@ ORBit_small_invoke_stub (CORBA_Object       obj,
 	adaptor_obj = obj->adaptor_obj;
 
 	if (adaptor_obj) {
-		tprintf_header (obj, m_data);
-		tprintf ("[in-proc]");
-		ORBit_small_handle_request (adaptor_obj, m_data->name, ret,
-					    args, ctx, NULL, ev);
-		goto clean_out;
-	}
+		/* FIXME: unchecked cast */
+		if (ORBit_poa_allow_cross_thread_call ((ORBit_POAObject) adaptor_obj)) {
+			tprintf_header (obj, m_data);
+			tprintf ("[in-proc]");
+			ORBit_small_handle_request (adaptor_obj, m_data->name, ret,
+						    args, ctx, NULL, ev);
+			goto clean_out;
+		} else {
+			tprintf ("[in-proc-XT]");
+			xt_proxy = ORBit_objref_get_proxy (obj);
+			obj = xt_proxy;
+		}
+	} else
+		giop_thread_new_check (NULL);
 
 	cnx = ORBit_object_get_connection (obj);
 
@@ -639,9 +638,11 @@ ORBit_small_invoke_stub (CORBA_Object       obj,
 		goto clean_out;
 	}
 
-	switch (orbit_small_demarshal (obj, &cnx, &mqe, ev,
-				       ret, m_data, args,
-				       RECV_BUFFER_RELEASE))
+	giop_recv_buffer_unuse (recv_buffer);
+	recv_buffer = giop_recv_buffer_get (&mqe);
+
+	switch (orbit_small_demarshal (obj, &cnx, recv_buffer, ev,
+				       ret, m_data, args))
 	{
 	case MARSHAL_SYS_EXCEPTION_COMPLETE:
 		completion_status = CORBA_COMPLETED_YES;
@@ -666,9 +667,12 @@ ORBit_small_invoke_stub (CORBA_Object       obj,
 	};
 
  clean_out:
+	ORBit_RootObject_release (xt_proxy);
+	giop_recv_buffer_unuse (recv_buffer);
+
 	tprintf_end_method ();
 	if (cnx)
-		linc_object_unref (cnx);
+		giop_connection_unref (cnx);
 	return;
 
  system_exception:
@@ -929,18 +933,19 @@ ORBit_small_invoke_adaptor (ORBit_OAObject     adaptor_obj,
 	giop_send_buffer_write (send_buffer, recv_buffer->connection, FALSE);
 	giop_send_buffer_unuse (send_buffer);
 
-	if (m_data->ret && tc->kind != CORBA_tk_void &&
-	    ev->_major == CORBA_NO_EXCEPTION) {
+	if (m_data->ret && tc->kind != CORBA_tk_void) {
 		switch (m_data->ret->kind) {
 		case BASE_TYPES:
 			break;
 		case CORBA_tk_objref:
 		case CORBA_tk_TypeCode:
-			CORBA_Object_release (*(CORBA_Object *) retval, ev);
+			if (ev->_major == CORBA_NO_EXCEPTION)
+				CORBA_Object_release (*(CORBA_Object *) retval, ev);
 			break;
 		case CORBA_tk_string:
 		case CORBA_tk_wstring:
-			ORBit_free (*(char **) retval);
+			if (ev->_major == CORBA_NO_EXCEPTION)
+				ORBit_free (*(char **) retval);
 			break;
 		case STRUCT_UNION_TYPES:
 			if (m_data->flags & ORBit_I_COMMON_FIXED_SIZE) {
@@ -948,7 +953,8 @@ ORBit_small_invoke_adaptor (ORBit_OAObject     adaptor_obj,
 				break;
 			} /* drop through */
 		default:
-			ORBit_free (pretval);
+			if (ev->_major == CORBA_NO_EXCEPTION)
+				ORBit_free (pretval);
 			break;
 		}
 	}
@@ -989,20 +995,30 @@ ORBit_small_invoke_adaptor (ORBit_OAObject     adaptor_obj,
 				ORBit_free (args [i]);
 				break;
 			}
-		} else if (ev->_major == CORBA_NO_EXCEPTION) { /* Out */
+		} else { /* Out */
 			switch (tc->kind) {
 			case BASE_TYPES:
 				break;
 			case CORBA_tk_objref:
 			case CORBA_tk_TypeCode:
-				CORBA_Object_release (*(CORBA_Object *) scratch [i], ev);
+				if (ev->_major == CORBA_NO_EXCEPTION)
+					CORBA_Object_release (*(CORBA_Object *) scratch [i], ev);
 				break;
 			case CORBA_tk_string:
 			case CORBA_tk_wstring:
-				ORBit_free (*(char **) scratch [i]);
+				if (ev->_major == CORBA_NO_EXCEPTION)
+					ORBit_free (*(char **) scratch [i]);
 				break;
+			case STRUCT_UNION_TYPES:
+			case CORBA_tk_array:
+				if (a->flags & ORBit_I_COMMON_FIXED_SIZE) {
+					ORBit_free (scratch [i]);
+					break;
+				}
+				/* drop through */
 			default:
-				ORBit_free (scratch [i]);
+				if (ev->_major == CORBA_NO_EXCEPTION)
+					ORBit_free (scratch [i]);
 				break;
 			}
 		}
@@ -1049,8 +1065,10 @@ ORBit_small_demarshal_async (ORBitAsyncQueueEntry *aqe,
 			     gpointer             *args,
 			     CORBA_Environment    *ev)
 {
-	switch (orbit_small_demarshal (aqe->obj, &aqe->mqe.cnx, &aqe->mqe, ev,
-				       ret, aqe->m_data, args, RECV_BUFFER_KEEP)) {
+	g_return_if_fail (aqe->mqe.buffer != NULL);
+
+	switch (orbit_small_demarshal (aqe->obj, &aqe->mqe.cnx, aqe->mqe.buffer, ev,
+				       ret, aqe->m_data, args)) {
 	case MARSHAL_SYS_EXCEPTION_COMPLETE:
 		aqe->completion_status = CORBA_COMPLETED_YES;
 		dprintf (MESSAGES, "Sys exception completed on id 0x%x\n\n",
@@ -1099,7 +1117,7 @@ async_recv_cb (ORBitAsyncQueueEntry *aqe)
 	aqe->mqe.async_cb = NULL;
 
 	if (!aqe->mqe.cnx ||
-	    aqe->mqe.cnx->parent.status == LINC_DISCONNECTED)
+	    aqe->mqe.cnx->parent.status == LINK_DISCONNECTED)
 		CORBA_exception_set_system (ev, ex_CORBA_COMM_FAILURE,
 					    aqe->completion_status);
 
@@ -1108,6 +1126,7 @@ async_recv_cb (ORBitAsyncQueueEntry *aqe)
 
 	ORBit_RootObject_release (aqe->obj);
 /*	ORBit_RootObject_release (aqe->m_data); */
+	giop_recv_list_destroy_queue_entry (&aqe->mqe);
 	g_free (aqe);
 	CORBA_exception_free (ev);
 }
@@ -1178,7 +1197,7 @@ ORBit_small_invoke_async (CORBA_Object         obj,
 
  clean_out:
 	if (cnx)
-		linc_object_unref (cnx);
+		giop_connection_unref (cnx);
 	tprintf_end_method ();
 	return;
 
@@ -1215,11 +1234,11 @@ get_status (GIOPConnection *cnx)
 
 	g_return_val_if_fail (cnx != NULL, ORBIT_CONNECTION_DISCONNECTED);
 
-	switch (linc_connection_get_status (LINC_CONNECTION (cnx))) {
-	case LINC_CONNECTED:
+	switch (link_connection_get_status (LINK_CONNECTION (cnx))) {
+	case LINK_CONNECTED:
 		ret = ORBIT_CONNECTION_CONNECTED;
 		break;
-	case LINC_CONNECTING:
+	case LINK_CONNECTING:
 		ret = ORBIT_CONNECTION_CONNECTED;
 		break;
 	default:
@@ -1234,6 +1253,9 @@ ORBit_small_get_connection_status (CORBA_Object obj)
 {
 	ORBitConnectionStatus ret;
 
+	g_return_val_if_fail (obj != CORBA_OBJECT_NIL,
+			      ORBIT_CONNECTION_DISCONNECTED);
+
 	if (ORBit_small_get_servant (obj))
 		ret = ORBIT_CONNECTION_IN_PROC;
 	else {
@@ -1243,7 +1265,7 @@ ORBit_small_get_connection_status (CORBA_Object obj)
 
 		if (cnx) {
 			ret = get_status (cnx);
-			linc_object_unref (cnx);
+			giop_connection_unref (cnx);
 		} else
 			ret = ORBIT_CONNECTION_DISCONNECTED;
 	}
@@ -1271,8 +1293,10 @@ ORBit_small_listen_for_broken (CORBA_Object obj,
 
 		if (cnx) {
 			ret = get_status (cnx);
-			g_signal_connect (cnx, "broken", fn, user_data);
-			linc_object_unref (cnx);
+			link_connection_add_broken_cb
+				(LINK_CONNECTION (cnx),
+				 (LinkBrokenCallback)fn, user_data);
+			giop_connection_unref (cnx);
 		} else
 			ret = ORBIT_CONNECTION_DISCONNECTED;
 	}
@@ -1281,8 +1305,9 @@ ORBit_small_listen_for_broken (CORBA_Object obj,
 }
 
 ORBitConnectionStatus
-ORBit_small_unlisten_for_broken (CORBA_Object obj,
-				 GCallback    fn)
+ORBit_small_unlisten_for_broken_full (CORBA_Object obj,
+				      GCallback    fn,
+				      gpointer     user_data)
 {
 	ORBitConnectionStatus ret;
 
@@ -1299,10 +1324,10 @@ ORBit_small_unlisten_for_broken (CORBA_Object obj,
 
 		if (cnx) {
 			ret = get_status (cnx);
-			g_signal_handlers_disconnect_matched (
-				cnx, G_SIGNAL_MATCH_FUNC,
-				0, 0, NULL, G_CALLBACK (fn), NULL);
-			linc_object_unref (cnx);
+			link_connection_remove_broken_cb
+				(LINK_CONNECTION (cnx),
+				 (LinkBrokenCallback)fn, user_data);
+			giop_connection_unref (cnx);
 		} else
 			ret = ORBIT_CONNECTION_DISCONNECTED;
 	}
@@ -1310,6 +1335,13 @@ ORBit_small_unlisten_for_broken (CORBA_Object obj,
 	return ret;
 }
 
+ORBitConnectionStatus
+ORBit_small_unlisten_for_broken (CORBA_Object obj,
+				 GCallback    fn)
+{
+	return ORBit_small_unlisten_for_broken_full (obj, fn, NULL);
+}
+
 ORBitConnection *
 ORBit_small_get_connection_ref (CORBA_Object obj)
 {
@@ -1333,17 +1365,16 @@ void
 ORBit_small_connection_unref (ORBitConnection *cnx)
 {
 	if (cnx)
-		linc_object_unref (cnx);
+		giop_connection_unref (cnx);
 }
 
 void
 ORBit_connection_set_max_buffer (ORBitConnection *cnx,
 				 gulong           max_buffer_bytes)
 {
-	LINCConnection *lcnx = (LINCConnection *) cnx;
+	LinkConnection *lcnx = (LinkConnection *) cnx;
 
-	g_return_if_fail (LINC_IS_CONNECTION (lcnx));
+	g_return_if_fail (LINK_IS_CONNECTION (lcnx));
 
-	linc_connection_set_max_buffer (lcnx, max_buffer_bytes);
+	link_connection_set_max_buffer (lcnx, max_buffer_bytes);
 }
-
