@@ -400,21 +400,18 @@ ORBit_POA_destroy_T_R (PortableServer_POA  poa,
 	for (i = 0; i < adaptors->len; i++) {
 		PortableServer_POA cpoa = g_ptr_array_index (adaptors, i);
 
-
-		LINK_MUTEX_UNLOCK (ORBit_RootObject_lifecycle_lock);
-
 		if (cpoa && cpoa != poa) {
-			ORBit_RootObject_duplicate (cpoa);
+			ORBit_RootObject_duplicate_T (cpoa);
+			LINK_MUTEX_UNLOCK (ORBit_RootObject_lifecycle_lock);
 			POA_LOCK (cpoa);
 
 			if (cpoa->parent_poa == poa) 
 				ORBit_POA_destroy_T_R (cpoa, etherealize_objects, ev);
 
 			POA_UNLOCK (cpoa);
-			ORBit_RootObject_release (cpoa);
+			LINK_MUTEX_LOCK (ORBit_RootObject_lifecycle_lock);
+			ORBit_RootObject_release_T (cpoa);
 		}
-
-		LINK_MUTEX_LOCK (ORBit_RootObject_lifecycle_lock);
 	}
 
 	POA_LOCK (poa);
@@ -1491,6 +1488,10 @@ ORBit_POA_handle_request (PortableServer_POA poa,
 				
 				switch (adaptor->thread_hint) {
 				case ORBIT_THREAD_HINT_PER_OBJECT:
+					/* Note: If the user bound this object to a specific
+					 * thread this thread will be called, otherwise a
+					 * new thread will be created for handling requests
+					 * to the object. */
 					pool_push_request_for_T (pobj, &pobj, &recv_buffer);
 					break;
 
